/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 *  @{
 */

/** @file
 *  
 */

#ifndef __NET_SOCKADDR_H__
#define __NET_SOCKADDR_H__

#include <sys/types.h>

#define INET_ADDRSTRLEN		( 4 * 3 + 3 )
#define INET6_ADDRSTRLEN	( 8 * 4 + 7 )

typedef struct sockaddr		sockaddr_t;
typedef struct in_addr		in_addr_t;
typedef struct sockaddr_in	sockaddr_in_t;
typedef struct sockaddr_in6	sockaddr_in6_t;
typedef struct addrinfo		addrinfo_t;

int	inet_ntop( uint16_t family, const uint8_t * data, char * address, size_t length );
int	inet_pton( uint16_t family, const char * address, uint8_t * data );

struct sockaddr{
	uint16_t		sa_family;		// address family, AF_xxx
	uint8_t			sa_data[ 14 ];	// 14 bytes of protocol address
};

struct in_addr{
	uint32_t		s_addr;			// that's a 32-bit int (4 bytes)
};

struct sockaddr_in{
	uint16_t		sin_family;		// Address family, AF_INET
	uint16_t		sin_port;		// Port number
	struct in_addr	sin_addr;		// Internet address
	uint8_t			sin_zero[ 8 ];	// Same size as struct sockaddr
};

struct in6_addr{
	unsigned char	s6_addr[ 16 ];	// IPv6 address
};

struct sockaddr_in6{
	uint16_t		sin6_family;	// address family, AF_INET6
	uint16_t		sin6_port;		// port number, Network Byte Order
	uint32_t		sin6_flowinfo;	// IPv6 flow information
	struct in6_addr	sin6_addr;		// IPv6 address
	uint32_t		sin6_scope_id;	// Scope ID
};

struct addrinfo{
	int				ai_flags;		// AI_PASSIVE, AI_CANONNAME, etc.
	uint16_t		ai_family;		// AF_INET, AF_INET6, AF_UNSPEC
	int				ai_socktype;	// SOCK_STREAM, SOCK_DGRAM
	int				ai_protocol;	// use 0 for "any"
	size_t			ai_addrlen;		// size of ai_addr in bytes
	struct sockaddr *	ai_addr;	// struct sockaddr_in or _in6
	char *			ai_canonname;	// full canonical hostname
	struct addrinfo *	ai_next;	// linked list, next node
};

/*int getaddrinfo(const char *node, // e.g. "www.example.com" or IP
const char *service, // e.g. "http" or port number
const struct addrinfo *hints,
struct addrinfo **res);

getnameinfo
*/

#endif

/** @}
 */
